#include "pot.hpp"

Pot::Pot(int x, int y, int width, int height,
	 int min_x, int min_y, int max_x, int max_y,
	 float min_cursor, float max_cursor, char* pot_units,
	 int cursor_width, int cursor_height,
	 char* name, float* initial_setting, 
	 void (*activate)(),
	 COLOR_TYPE shade_clr, COLOR_TYPE border_clr,
	 COLOR_TYPE inactive_clr, COLOR_TYPE active_clr,
	 COLOR_TYPE legend_clr) :

	 Icon(x, y, x + width, y + height)
{
	minimum = min_cursor;
	maximum = max_cursor;
	units = pot_units;

	// make a copy of the reference variable
	// and the activation function, which will
	// invoked when the pot is used
	strncpy(legend, name, sizeof(legend) - 1);
	pot_setting = initial_setting;
	activation_function = activate;

	shade_color = shade_clr;
	border_color = border_clr;
	legend_color = legend_clr;
	fill_color = legend_backdrop_color = 
		inactive_color = inactive_clr;
	active_color = active_clr;

	pot_box = new ShadedBox(x1, y1,
				 x2 - x1,
				 y2 - y1,
				 shade_color,
				 border_color,
				 fill_color);

	cursor = new Cursor(this, cursor_width, cursor_height,
			    min_x, min_y,
			    max_x, max_y,
			    fill_color);
}

void Pot::Display()
{

	pot_box->Display();

	// subdivide pot into areas
	BOX_TYPE box;
	box.x1 = x1 + 1;
	box.y1 = y1 + POT_LEGEND_HEIGHT + 1;
	box.x2 = x2 - 2;
	box.y2 = y2 - POT_LEGEND_HEIGHT - 2;
	GraphicsHandler->DrawBox(border_color, 
						SOLID_LINE, &box);

	// center title on pot
	box.x1 = x1 + 1;
	box.y1 = y1;
	box.x2 = x2 - 2;
	box.y2 = y1 + POT_LEGEND_HEIGHT;
	Legend pot_legend;
	pot_legend.Center(&box, legend_color, legend);

	// display the cursor controls
	cursor->Display();
	SettingDisplay();
}


void Pot::Setting(float value)
{
	// update the pot's setting variable
	*pot_setting = value;

	// invoke the application program to
	// handle the pot
	if (activation_function)
	  (*activation_function)();
}

void Pot::SettingDisplay()
{
	// erase previous string
	BOX_TYPE box;
	box.x1 = x1 + 3;
	box.y1 = y2 - POT_LEGEND_HEIGHT;
	box.x2 = x2 - 4;
	box.y2 = y2 - 6;
	GraphicsHandler->FillBox(legend_backdrop_color, &box);

	// convert the setting from range (0..1) to the proper
	// range for this pot
	float displayed_value = \
		*pot_setting * (maximum - minimum) + minimum;

	// display the new setting, centered
	char text [40];
	sprintf(text, "%.1f %s", displayed_value, units);
	Legend pot_setting;
	pot_setting.Center(&box, legend_color, text);
}

Pot::~Pot()
{
	delete pot_box;
	delete cursor;
}



Cursor::Cursor(Pot* parent_pot,
	       int cwidth, int cheight,
	       int min_x, int min_y, int max_x, int max_y,
	       COLOR_TYPE background_clr) :

    Icon(-1, -1, -1, -1)	
{
	// save a reference to our parent potentiometer
	parent = parent_pot;

	// maintain a copy of the cursor dimensions
	width = cwidth;
	height = cheight;

	// ... and the limits of motion
	minimum_x = min_x;
	minimum_y = min_y;
	maximum_x = max_x;
	maximum_y = max_y;

	// get the initial setting of the potentiometer
	float value = parent->Setting();

	// see whether cursor moves horizontally or vertically
	// and compute the initial screen location of the
	// cursor
	if (height > width) {
	  // it moves horizontally
	    x1 = minimum_x + value * (maximum_x - minimum_x);
	    y1 = minimum_y;
	}
	else {
	  // it moves vertically
	  x1 = minimum_x;
	  y1 = minimum_y + value * (maximum_y - minimum_y);
	}

	// update the cursor coordinates
	x2 = x1 + width;
	y2 = y1 + height;

	// record the color attributes
	shade_color = BLACK;
	border_color = RED;
	fill_color = GRAY;
	background_color = background_clr;
}

void Cursor::Display()
{
	// fill inside of cursor
	BOX_TYPE box;
	box.x1 = x1;
	box.y1 = y1 + 1;
	box.x2 = x2 - 1;
	box.y2 = y2 - 1;
	GraphicsHandler->FillBox(fill_color, &box);

	// highlight top of cursor
	box.x1 = x1;
	box.y1 = y2;
	box.x2 = x2 - 1;
	box.y2 = y2;
	GraphicsHandler->FillBox(WHITE, &box);

	// highlight left side of cursor
	box.x1 = x1;
	box.y1 = y1 + 2;
	box.x2 = x1;
	box.y2 = y2;
	GraphicsHandler->FillBox(WHITE, &box);

	// shade right side of cursor
	box.x1 = x2;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2 - 1;
	GraphicsHandler->FillBox(shade_color, &box);

	// shade bottom of cursor
	box.x1 = x1 + 1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y1 + 1;
	GraphicsHandler->FillBox(shade_color, &box);
}

void Cursor::Blank()
{
	BOX_TYPE box;

	// blank out the cursor, making it invisible
	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	GraphicsHandler->FillBox(background_color, &box);
}

void Cursor::DisplayOutline()
{
	BOX_TYPE box;

	// repaint the cursor, showing it as a
	// dotted outline
	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	GraphicsHandler->DrawBox(WHITE, DOTTED_LINE, &box);
}

void Cursor::UnDisplayOutline()
{
	BOX_TYPE box;

	// erase the dotted outline
	box.x1 = x1;
	box.y1 = y1;
	box.x2 = x2;
	box.y2 = y2;
	GraphicsHandler->DrawBox(background_color, 
						DOTTED_LINE, &box);
}

void Cursor::LeftButtonPressed()
{
	// repaint the entire parent, in case
	// it was partially obscured by an icon.
	parent->Display();

	// erase the cursor
	Blank();

	// and replace it with the dotted outline
	DisplayOutline();
	EventsHandler->ShowMouseCursor();
}

void Cursor::LeftButtonReleased()
{
	UnDisplayOutline();
	EventsHandler->HideMouseCursor();
	Display();
	EventsHandler->ShowMouseCursor();
}

void Cursor::LeftButtonDragged(unsigned x, unsigned y, 
						 int status)
{
	int new_x, new_y;

	// see if it moved in the X direction
	if (x) {
	  new_x = x1 + x;
	  if ( (new_x >= minimum_x) && (new_x <= maximum_x) ) {
	    EventsHandler->HideMouseCursor();
	    UnDisplayOutline();
	    x1 = new_x;
	    x2 = x1 + width;
	    DisplayOutline();
	    parent->Setting( \
		(float) (new_x - minimum_x) / \
			(maximum_x - minimum_x) );
	    parent->SettingDisplay();
	    EventsHandler->ShowMouseCursor();
	  }
	}

	// see if it moved in the Y direction
	if (y) {
	  new_y = y1 + y;
	  if ( (new_y >= minimum_y) && (new_y <= maximum_y) ) {
	    EventsHandler->HideMouseCursor();
	    UnDisplayOutline();
	    y1 = new_y;
	    y2 = y1 + height;
	    DisplayOutline();
	    parent->Setting( \
		(float) (new_y - minimum_y) / \
			(maximum_y - minimum_y) );
	    parent->SettingDisplay();
	    EventsHandler->ShowMouseCursor();
	  }
	}
}



VerticalPot::VerticalPot(int x, int y,
		    float min, float max, char* units,
		    char* name, float* value, 
		    void (*activate)() ) :

	 Pot(x, y,
	 VERTICAL_POT_WIDTH, VERTICAL_POT_HEIGHT,
	 x + 2,
	 y + POT_LEGEND_HEIGHT + 2,
	 x + 2,
	 y + VERTICAL_POT_HEIGHT - POT_LEGEND_HEIGHT - \
	 3 - VERTICAL_POT_CURSOR_HEIGHT,
	 min, max, units,
	 VERTICAL_POT_CURSOR_WIDTH,
	 VERTICAL_POT_CURSOR_HEIGHT,
	 name, value, activate,
	 BLACK,
	 BRIGHT_GREEN,
	 BRIGHT_BLUE,
	 BRIGHT_CYAN,
	 WHITE)
{}


AttenuatorPot::AttenuatorPot(int x, int y, char* l, 
			float* v, void (*f)() ) :
	VerticalPot(x, y, -102, 0, "dB", l, v, f)
{}


ToneLevelPot::ToneLevelPot(int x, int y, char* l, 
			float* v, void (*f)() ) :

	VerticalPot(x, y, TONE_MIN_LEVEL, 
			  TONE_MAX_LEVEL, "dB", l, v, f)
{}


HorizontalPot::HorizontalPot(int x, int y,
		      float min, float max, char* units,
		      char* name, float* value, 
			 void (*activate)() ) :

	 Pot(x, y,
	 HORIZONTAL_POT_WIDTH, HORIZONTAL_POT_HEIGHT,
	 x + 2,
	 y + POT_LEGEND_HEIGHT + 2,
	 x + HORIZONTAL_POT_WIDTH - \
		HORIZONTAL_POT_CURSOR_WIDTH - 3,
	 y + POT_LEGEND_HEIGHT + 2,
	 min, max, units,
	 HORIZONTAL_POT_CURSOR_WIDTH,
	 HORIZONTAL_POT_CURSOR_HEIGHT,
	 name, value, activate,
	 BLACK,
	 BRIGHT_GREEN,
	 BRIGHT_BLUE,
	 BRIGHT_CYAN,
	 WHITE)
{}
